@@ -1,7 +1,6 @@
-import asyncio
 import logging
 from asyncio import AbstractEventLoop
-from typing import Any, Dict, Text, Optional, Union
+from typing import Any, Dict, Text, Optional, Union, TypeVar, Type
 
 import aiormq
 
@@ -13,6 +12,9 @@ from rasa.utils.endpoints import EndpointConfig
 logger = logging.getLogger(__name__)
 
 
+EB = TypeVar("EB", bound="EventBroker")
+
+
 class EventBroker:
     """Base class for any event broker implementation."""
 
@@ -40,10 +42,10 @@ class EventBroker:
 
     @classmethod
     async def from_endpoint_config(
-        cls,
+        cls: Type[EB],
         broker_config: EndpointConfig,
         event_loop: Optional[AbstractEventLoop] = None,
-    ) -> "EventBroker":
+    ) -> Optional[EB]:
         """Creates an `EventBroker` from the endpoint configuration.
 
         Args:
@@ -114,16 +116,6 @@ async def _load_from_module_name_in_endpoint_config(
         event_broker_class = rasa.shared.utils.common.class_from_module_path(
             broker_config.type
         )
-        if not asyncio.iscoroutinefunction(event_broker_class.from_endpoint_config):
-            rasa.shared.utils.io.raise_deprecation_warning(
-                f"The method "
-                f"'{EventBroker.__name__}.{EventBroker.from_endpoint_config.__name__}' "
-                f"was changed to be 'async'. Your event broker '{broker_config.type}' "
-                f"is currently synchronous. Support for synchronous implementations of "
-                f"'{EventBroker.from_endpoint_config.__name__}' will be removed in "
-                f"3.0.0."
-            )
-            return event_broker_class.from_endpoint_config(broker_config)
         return await event_broker_class.from_endpoint_config(broker_config)
     except (AttributeError, ImportError) as e:
         logger.warning(
