@@ -1,8 +1,10 @@
+from __future__ import annotations
+import abc
 import copy
-import json
 import logging
 from enum import Enum
 from pathlib import Path
+from rasa.shared.core.events import Event
 from typing import (
     Any,
     List,
@@ -10,37 +12,36 @@ from typing import (
     Text,
     Dict,
     Callable,
-    Type,
     Union,
     Tuple,
     TYPE_CHECKING,
 )
+
 import numpy as np
 
-from rasa.core.exceptions import UnsupportedDialogueModelError
-from rasa.shared.core.events import Event
-
-import rasa.shared.utils.common
+from rasa.engine.graph import GraphComponent, ExecutionContext
+from rasa.engine.storage.resource import Resource
+from rasa.engine.storage.storage import ModelStorage
+from rasa.core.featurizers.precomputation import MessageContainerForCoreFeaturization
+from rasa.core.featurizers.tracker_featurizers import TrackerFeaturizer
+from rasa.core.featurizers.tracker_featurizers import MaxHistoryTrackerFeaturizer
+from rasa.core.featurizers.single_state_featurizer import SingleStateFeaturizer
+from rasa.core.featurizers.tracker_featurizers import FEATURIZER_FILE
 import rasa.utils.common
 import rasa.shared.utils.io
+from rasa.shared.exceptions import RasaException, FileIOException
+from rasa.shared.nlu.constants import ENTITIES, INTENT, TEXT, ACTION_TEXT, ACTION_NAME
 from rasa.shared.core.domain import Domain, State
-from rasa.core.featurizers.single_state_featurizer import SingleStateFeaturizer
-from rasa.core.featurizers.tracker_featurizers import (
-    TrackerFeaturizer,
-    MaxHistoryTrackerFeaturizer,
-    FEATURIZER_FILE,
-)
-from rasa.shared.nlu.interpreter import NaturalLanguageInterpreter
 from rasa.shared.core.trackers import DialogueStateTracker
 from rasa.shared.core.generator import TrackerWithCachedStates
-from rasa.core.constants import DEFAULT_POLICY_PRIORITY
-from rasa.shared.core.constants import (
-    USER,
-    SLOTS,
-    PREVIOUS_ACTION,
-    ACTIVE_LOOP,
+from rasa.core.constants import (
+    DEFAULT_POLICY_PRIORITY,
+    POLICY_PRIORITY,
+    POLICY_MAX_HISTORY,
 )
-from rasa.shared.nlu.constants import ENTITIES, INTENT, TEXT, ACTION_TEXT, ACTION_NAME
+from rasa.shared.core.constants import USER, SLOTS, PREVIOUS_ACTION, ACTIVE_LOOP
+import rasa.shared.utils.common
+
 
 if TYPE_CHECKING:
     from rasa.shared.nlu.training_data.features import Features
@@ -62,21 +63,19 @@ class SupportedData(Enum):
     ML_AND_RULE_DATA = 3
 
     @staticmethod
-    def trackers_for_policy(
-        policy: Union["Policy", Type["Policy"]],
+    def trackers_for_supported_data(
+        supported_data: SupportedData,
         trackers: Union[List[DialogueStateTracker], List[TrackerWithCachedStates]],
     ) -> Union[List[DialogueStateTracker], List[TrackerWithCachedStates]]:
         """Return trackers for a given policy.
 
         Args:
-            policy: Policy or policy type to return trackers for.
+            supported_data: Supported data filter for the `trackers`.
             trackers: Trackers to split.
 
         Returns:
             Trackers from ML-based training data and/or rule-based data.
         """
-        supported_data = policy.supported_data()
-
         if supported_data == SupportedData.RULE_DATA:
             return [tracker for tracker in trackers if tracker.is_rule_tracker]
 
@@ -87,7 +86,9 @@ class SupportedData(Enum):
         return trackers
 
 
-class Policy:
+class Policy(GraphComponent):
+    """Common parent class for all dialogue policies."""
+
     @staticmethod
     def supported_data() -> SupportedData:
         """The type of data supported by this policy.
@@ -100,41 +101,86 @@ class Policy:
         """
         return SupportedData.ML_DATA
 
-    @staticmethod
-    def _standard_featurizer() -> MaxHistoryTrackerFeaturizer:
-        return MaxHistoryTrackerFeaturizer(SingleStateFeaturizer())
-
-    @classmethod
-    def _create_featurizer(
-        cls, featurizer: Optional[TrackerFeaturizer] = None
-    ) -> TrackerFeaturizer:
-        if featurizer:
-            return copy.deepcopy(featurizer)
-        else:
-            return cls._standard_featurizer()
-
     def __init__(
         self,
+        config: Dict[Text, Any],
+        model_storage: ModelStorage,
+        resource: Resource,
+        execution_context: ExecutionContext,
         featurizer: Optional[TrackerFeaturizer] = None,
-        priority: int = DEFAULT_POLICY_PRIORITY,
-        should_finetune: bool = False,
-        **kwargs: Any,
     ) -> None:
         """Constructs a new Policy object."""
-        self.__featurizer = self._create_featurizer(featurizer)
-        self.priority = priority
-        self.finetune_mode = should_finetune
-        self._rule_only_data = {}
+        self.config = config
+        if featurizer is None:
+            featurizer = self._create_featurizer()
+        self.__featurizer = featurizer
+
+        self.priority = config.get(POLICY_PRIORITY, DEFAULT_POLICY_PRIORITY)
+        self.finetune_mode = execution_context.is_finetuning
+
+        self._model_storage = model_storage
+        self._resource = resource
+
+    @classmethod
+    def create(
+        cls,
+        config: Dict[Text, Any],
+        model_storage: ModelStorage,
+        resource: Resource,
+        execution_context: ExecutionContext,
+        **kwargs: Any,
+    ) -> Policy:
+        """Creates a new untrained policy (see parent class for full docstring)."""
+        return cls(config, model_storage, resource, execution_context)
+
+    def _create_featurizer(self) -> TrackerFeaturizer:
+        policy_config = copy.deepcopy(self.config)
+
+        featurizer_configs = policy_config.get("featurizer")
+
+        if not featurizer_configs:
+            return self._standard_featurizer()
+
+        featurizer_func = _get_featurizer_from_config(
+            featurizer_configs,
+            self.__class__.__name__,
+            lookup_path="rasa.core.featurizers.tracker_featurizers",
+        )
+        featurizer_config = featurizer_configs[0]
+
+        state_featurizer_configs = featurizer_config.pop("state_featurizer", None)
+        if state_featurizer_configs:
+            state_featurizer_func = _get_featurizer_from_config(
+                state_featurizer_configs,
+                self.__class__.__name__,
+                lookup_path="rasa.core.featurizers.single_state_featurizer",
+            )
+            state_featurizer_config = state_featurizer_configs[0]
+
+            featurizer_config["state_featurizer"] = state_featurizer_func(
+                **state_featurizer_config
+            )
+
+        featurizer = featurizer_func(**featurizer_config)
+        if (
+            isinstance(featurizer, MaxHistoryTrackerFeaturizer)
+            and POLICY_MAX_HISTORY in policy_config
+            and POLICY_MAX_HISTORY not in featurizer_config
+        ):
+            featurizer.max_history = policy_config[POLICY_MAX_HISTORY]
+        return featurizer
+
+    def _standard_featurizer(self) -> MaxHistoryTrackerFeaturizer:
+        """Initializes the standard featurizer for this policy."""
+        return MaxHistoryTrackerFeaturizer(
+            SingleStateFeaturizer(), self.config.get(POLICY_MAX_HISTORY)
+        )
 
     @property
     def featurizer(self) -> TrackerFeaturizer:
         """Returns the policy's featurizer."""
         return self.__featurizer
 
-    def set_shared_policy_states(self, **kwargs: Any) -> None:
-        """Sets policy's shared states for correct featurization."""
-        self._rule_only_data = kwargs.get("rule_only_data", {})
-
     @staticmethod
     def _get_valid_params(func: Callable, **kwargs: Any) -> Dict:
         """Filters out kwargs that cannot be passed to func.
@@ -158,13 +204,13 @@ class Policy:
         self,
         training_trackers: List[DialogueStateTracker],
         domain: Domain,
-        interpreter: NaturalLanguageInterpreter,
+        precomputations: Optional[MessageContainerForCoreFeaturization],
         bilou_tagging: bool = False,
         **kwargs: Any,
     ) -> Tuple[
-        List[List[Dict[Text, List["Features"]]]],
+        List[List[Dict[Text, List[Features]]]],
         np.ndarray,
-        List[List[Dict[Text, List["Features"]]]],
+        List[List[Dict[Text, List[Features]]]],
     ]:
         """Transform training trackers into a vector representation.
 
@@ -175,7 +221,7 @@ class Policy:
             training_trackers:
                 the list of the :class:`rasa.core.trackers.DialogueStateTracker`
             domain: the :class:`rasa.shared.core.domain.Domain`
-            interpreter: the :class:`rasa.core.interpreter.NaturalLanguageInterpreter`
+            precomputations: Contains precomputed features and attributes.
             bilou_tagging: indicates whether BILOU tagging should be used or not
 
         Returns:
@@ -191,8 +237,8 @@ class Policy:
         state_features, label_ids, entity_tags = self.featurizer.featurize_trackers(
             training_trackers,
             domain,
-            interpreter,
-            bilou_tagging,
+            precomputations=precomputations,
+            bilou_tagging=bilou_tagging,
             ignore_action_unlikely_intent=self.supported_data()
             == SupportedData.ML_DATA,
         )
@@ -214,6 +260,7 @@ class Policy:
         tracker: DialogueStateTracker,
         domain: Domain,
         use_text_for_last_user_input: bool = False,
+        rule_only_data: Optional[Dict[Text, Any]] = None,
     ) -> List[State]:
         """Transforms tracker to states for prediction.
 
@@ -222,6 +269,8 @@ class Policy:
             domain: The Domain.
             use_text_for_last_user_input: Indicates whether to use text or intent label
                 for featurizing last user input.
+            rule_only_data: Slots and loops which are specific to rules and hence
+                should be ignored by this policy.
 
         Returns:
             A list of states.
@@ -231,7 +280,7 @@ class Policy:
             domain,
             use_text_for_last_user_input=use_text_for_last_user_input,
             ignore_rule_only_turns=self.supported_data() == SupportedData.ML_DATA,
-            rule_only_data=self._rule_only_data,
+            rule_only_data=rule_only_data,
             ignore_action_unlikely_intent=self.supported_data()
             == SupportedData.ML_DATA,
         )[0]
@@ -240,9 +289,10 @@ class Policy:
         self,
         tracker: DialogueStateTracker,
         domain: Domain,
-        interpreter: NaturalLanguageInterpreter,
+        precomputations: Optional[MessageContainerForCoreFeaturization],
+        rule_only_data: Optional[Dict[Text, Any]],
         use_text_for_last_user_input: bool = False,
-    ) -> List[List[Dict[Text, List["Features"]]]]:
+    ) -> List[List[Dict[Text, List[Features]]]]:
         """Transforms training tracker into a vector representation.
 
         The trackers, consisting of multiple turns, will be transformed
@@ -251,9 +301,11 @@ class Policy:
         Args:
             tracker: The tracker to be featurized.
             domain: The Domain.
-            interpreter: The NLU interpreter.
+            precomputations: Contains precomputed features and attributes.
             use_text_for_last_user_input: Indicates whether to use text or intent label
                 for featurizing last user input.
+            rule_only_data: Slots and loops which are specific to rules and hence
+                should be ignored by this policy.
 
         Returns:
             A list (corresponds to the list of trackers)
@@ -265,48 +317,55 @@ class Policy:
         return self.featurizer.create_state_features(
             [tracker],
             domain,
-            interpreter,
+            precomputations=precomputations,
             use_text_for_last_user_input=use_text_for_last_user_input,
             ignore_rule_only_turns=self.supported_data() == SupportedData.ML_DATA,
-            rule_only_data=self._rule_only_data,
+            rule_only_data=rule_only_data,
             ignore_action_unlikely_intent=self.supported_data()
             == SupportedData.ML_DATA,
         )
 
+    @abc.abstractmethod
     def train(
         self,
         training_trackers: List[TrackerWithCachedStates],
         domain: Domain,
-        interpreter: NaturalLanguageInterpreter,
         **kwargs: Any,
-    ) -> None:
-        """Trains the policy on given training trackers.
+    ) -> Resource:
+        """Trains a policy.
 
         Args:
-            training_trackers:
-                the list of the :class:`rasa.core.trackers.DialogueStateTracker`
-            domain: the :class:`rasa.shared.core.domain.Domain`
-            interpreter: Interpreter which can be used by the polices for featurization.
+            training_trackers: The story and rules trackers from the training data.
+            domain: The model's domain.
+            **kwargs: Depending on the specified `needs` section and the resulting
+                graph structure the policy can use different input to train itself.
+
+        Returns:
+            A policy must return its resource locator so that potential children nodes
+            can load the policy from the resource.
         """
         raise NotImplementedError("Policy must have the capacity to train.")
 
+    @abc.abstractmethod
     def predict_action_probabilities(
         self,
         tracker: DialogueStateTracker,
         domain: Domain,
-        interpreter: NaturalLanguageInterpreter,
+        rule_only_data: Optional[Dict[Text, Any]] = None,
         **kwargs: Any,
-    ) -> "PolicyPrediction":
+    ) -> PolicyPrediction:
         """Predicts the next action the bot should take after seeing the tracker.
 
         Args:
-            tracker: the :class:`rasa.core.trackers.DialogueStateTracker`
-            domain: the :class:`rasa.shared.core.domain.Domain`
-            interpreter: Interpreter which may be used by the policies to create
-                additional features.
+            tracker: The tracker containing the conversation history up to now.
+            domain: The model's domain.
+            rule_only_data: Slots and loops which are specific to rules and hence
+                should be ignored by this policy.
+            **kwargs: Depending on the specified `needs` section and the resulting
+                graph structure the policy can use different input to make predictions.
 
         Returns:
-             The policy's prediction (e.g. the probabilities for the actions).
+             The prediction.
         """
         raise NotImplementedError("Policy must have the capacity to predict.")
 
@@ -319,7 +378,7 @@ class Policy:
         is_no_user_prediction: bool = False,
         diagnostic_data: Optional[Dict[Text, Any]] = None,
         action_metadata: Optional[Dict[Text, Any]] = None,
-    ) -> "PolicyPrediction":
+    ) -> PolicyPrediction:
         return PolicyPrediction(
             probabilities,
             self.__class__.__name__,
@@ -332,89 +391,34 @@ class Policy:
             action_metadata=action_metadata,
         )
 
-    def _metadata(self) -> Optional[Dict[Text, Any]]:
-        """Returns this policy's attributes that should be persisted.
-
-        Policies using the default `persist()` and `load()` implementations must
-        implement the `_metadata()` method."
-
-        Returns:
-            The policy metadata.
-        """
-        pass
-
     @classmethod
-    def _metadata_filename(cls) -> Optional[Text]:
-        """Returns the filename of the persisted policy metadata.
+    def load(
+        cls,
+        config: Dict[Text, Any],
+        model_storage: ModelStorage,
+        resource: Resource,
+        execution_context: ExecutionContext,
+        **kwargs: Any,
+    ) -> Policy:
+        """Loads a trained policy (see parent class for full docstring)."""
+        featurizer = None
 
-        Policies using the default `persist()` and `load()` implementations must
-        implement the `_metadata_filename()` method.
+        try:
+            with model_storage.read_from(resource) as path:
+                if (Path(path) / FEATURIZER_FILE).is_file():
+                    featurizer = TrackerFeaturizer.load(path)
 
-        Returns:
-            The filename of the persisted policy metadata.
-        """
-        pass
+                config.update(kwargs)
 
-    def persist(self, path: Union[Text, Path]) -> None:
-        """Persists the policy to storage.
+        except (ValueError, FileNotFoundError, FileIOException):
+            logger.debug(
+                f"Couldn't load metadata for policy '{cls.__name__}' as the persisted "
+                f"metadata couldn't be loaded."
+            )
 
-        Args:
-            path: Path to persist policy to.
-        """
-        # not all policies have a featurizer
-        if self.featurizer is not None:
-            self.featurizer.persist(path)
-
-        file = Path(path) / self._metadata_filename()
-
-        rasa.shared.utils.io.create_directory_for_file(file)
-        rasa.shared.utils.io.dump_obj_as_json_to_file(file, self._metadata())
-
-    @classmethod
-    def load(cls, path: Union[Text, Path], **kwargs: Any) -> "Policy":
-        """Loads a policy from path.
-
-        Args:
-            path: Path to load policy from.
-
-        Returns:
-            An instance of `Policy`.
-        """
-        metadata_file = Path(path) / cls._metadata_filename()
-
-        if metadata_file.is_file():
-            data = json.loads(rasa.shared.utils.io.read_file(metadata_file))
-
-            if (Path(path) / FEATURIZER_FILE).is_file():
-                featurizer = TrackerFeaturizer.load(path)
-                data["featurizer"] = featurizer
-
-            data.update(kwargs)
-
-            constructor_args = rasa.shared.utils.common.arguments_of(cls)
-            if "kwargs" not in constructor_args:
-                if set(data.keys()).issubset(set(constructor_args)):
-                    rasa.shared.utils.io.raise_deprecation_warning(
-                        f"`{cls.__name__}.__init__` does not accept `**kwargs` "
-                        f"This is required for contextual information e.g. the flag "
-                        f"`should_finetune`.",
-                        warn_until_version="3.0.0",
-                    )
-                else:
-                    raise UnsupportedDialogueModelError(
-                        f"`{cls.__name__}.__init__` does not accept `**kwargs`. "
-                        f"Attempting to pass {data} to the policy. "
-                        f"This argument should be added to all policies by "
-                        f"Rasa Open Source 3.0.0."
-                    )
-
-            return cls(**data)
-
-        logger.info(
-            f"Couldn't load metadata for policy '{cls.__name__}'. "
-            f"File '{metadata_file}' doesn't exist."
+        return cls(
+            config, model_storage, resource, execution_context, featurizer=featurizer
         )
-        return cls()
 
     def _default_predictions(self, domain: Domain) -> List[float]:
         """Creates a list of zeros.
@@ -476,6 +480,10 @@ class Policy:
 
         return "\n".join(formatted_states)
 
+    def __repr__(self) -> Text:
+        """Returns text representation of object."""
+        return f"{self.__class__.__name__}@{id(self)}"
+
 
 class PolicyPrediction:
     """Stores information about the prediction of a `Policy`."""
@@ -574,7 +582,7 @@ class PolicyPrediction:
             and self.policy_name == other.policy_name
             and self.policy_priority == other.policy_priority
             and self.events == other.events
-            and self.optional_events == other.events
+            and self.optional_events == other.optional_events
             and self.is_end_to_end_prediction == other.is_end_to_end_prediction
             and self.is_no_user_prediction == other.is_no_user_prediction
             and self.hide_rule_turn == other.hide_rule_turn
@@ -594,10 +602,10 @@ class PolicyPrediction:
 
     @property
     def max_confidence(self) -> float:
-        """Gets the highest predicted probability.
+        """Gets the highest predicted confidence.
 
         Returns:
-            The highest predicted probability.
+            The highest predicted confidence.
         """
         return max(self.probabilities, default=0.0)
 
@@ -620,3 +628,30 @@ def confidence_scores_for(
     results[idx] = value
 
     return results
+
+
+class InvalidPolicyConfig(RasaException):
+    """Exception that can be raised when policy config is not valid."""
+
+
+def _get_featurizer_from_config(
+    config: List[Dict[Text, Any]], policy_name: Text, lookup_path: Text
+) -> Callable[..., TrackerFeaturizer]:
+    """Gets the featurizer initializer and its arguments from a policy config."""
+    # Only 1 featurizer is allowed
+    if len(config) > 1:
+        featurizer_names = [
+            featurizer_config.get("name") for featurizer_config in config
+        ]
+        raise InvalidPolicyConfig(
+            f"Every policy can only have 1 featurizer but '{policy_name}' "
+            f"uses {len(config)} featurizers ('{', '.join(featurizer_names)}')."
+        )
+
+    featurizer_config = config[0]
+    featurizer_name = featurizer_config.pop("name")
+    featurizer_func = rasa.shared.utils.common.class_from_module_path(
+        featurizer_name, lookup_path=lookup_path
+    )
+
+    return featurizer_func
