@@ -1,8 +1,9 @@
+import asyncio
 import pprint as pretty_print
 import typing
 from typing import Any, Dict, Optional, Text
-import asyncio
 
+from rasa.shared.exceptions import RasaException
 from rasa.shared.utils.cli import print_success
 import rasa.core.agent
 import rasa.utils.common
@@ -31,6 +32,12 @@ def chat(
     if model_path:
         agent = rasa.core.agent.load_agent(model_path=model_path, endpoints=endpoints)
 
+    if agent is None:
+        raise RasaException(
+            "Either the provided model path could not load the agent "
+            "or no core agent was provided."
+        )
+
     print("Your bot is ready to talk! Type your messages here or send '/stop'.")
     while True:
         message = input()
