@@ -254,7 +254,7 @@ class RasaModelData:
         self.label_sub_key = label_sub_key
         # should be updated when features are added
         self.num_examples = self.number_of_examples()
-        self.sparse_feature_sizes = {}
+        self.sparse_feature_sizes: Dict[Text, Dict[Text, List[int]]] = {}
 
     def get(
         self, key: Text, sub_key: Optional[Text] = None
@@ -321,7 +321,7 @@ class RasaModelData:
         Returns:
             The simplified data.
         """
-        out_data = {}
+        out_data: Data = {}
         for key, attribute_data in self.data.items():
             out_data[key] = {}
             for sub_key, features in attribute_data.items():
@@ -417,7 +417,7 @@ class RasaModelData:
         units = 0
         for features in self.data[key][sub_key]:
             if len(features) > 0:
-                units += features.units
+                units += features.units  # type: ignore[operator]
 
         return units
 
@@ -569,7 +569,7 @@ class RasaModelData:
                 for data in attribute_data.values()
                 for v in data
             ]
-            solo_values = [
+            solo_values: List[Any] = [
                 []
                 for attribute_data in self.data.values()
                 for data in attribute_data.values()
@@ -695,7 +695,9 @@ class RasaModelData:
         # if a label was skipped in current batch
         skipped = [False] * num_label_ids
 
-        new_data = defaultdict(lambda: defaultdict(list))
+        new_data: defaultdict[
+            Text, defaultdict[Text, List[List[FeatureArray]]]
+        ] = defaultdict(lambda: defaultdict(list))
 
         while min(num_data_cycles) == 0:
             if shuffle:
@@ -846,8 +848,12 @@ class RasaModelData:
         Returns:
             The test and train RasaModelData
         """
-        data_train = defaultdict(lambda: defaultdict(list))
-        data_val = defaultdict(lambda: defaultdict(list))
+        data_train: defaultdict[
+            Text, defaultdict[Text, List[FeatureArray]]
+        ] = defaultdict(lambda: defaultdict(list))
+        data_val: defaultdict[Text, defaultdict[Text, List[Any]]] = defaultdict(
+            lambda: defaultdict(list)
+        )
 
         # output_values = x_train, x_val, y_train, y_val, z_train, z_val, etc.
         # order is kept, e.g. same order as model data keys
